<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.9.2.26 $
 * $Author: tora60 $
 * $Date: 2011/07/15 11:22:51 $
 */

/**
 * pmxHeader
 *
 * @package pragmaMx
 * @author tora60
 * @copyright Copyright (c) 2008
 * @version $Id: Header.php,v 1.9.2.26 2011/07/15 11:22:51 tora60 Exp $
 * @access public
 */
class pmxHeader {
    private static $__adds = array();

    private static $__urls = array();

    private static $__jquery = array('required' => false,
        'loaded' => false,
        'main' => 'jquery.js',
        'path' => 'includes/javascript/jquery/',
        'files' => array(),
        );

    /**
     * pmxHeader::__construct()
     * Ein private Konstruktor; verhindert die direkte Erzeugung des Objektes
     */
    private function __construct()
    {
    }

    /**
     * Returns content of the output
     *
     * @returns array
     * @static
     */
    public static function get()
    {
        $buf = pmxHeader::fetch();
        $buf = trim(implode("\n", $buf));
        /* unnoetige spaces entfernen */
        $buf = trim(preg_replace('#\s*\n\s+#', "\n", $buf));
        return $buf;
    }

    /**
     * Outputs content of the buffer and delete the buffer
     *
     * @returns string
     * @static
     */
    public static function show()
    {
        // Puffer ausgeben
        echo '<!-- start~header~' . MX_TIME . '~pmx -->', self::get(), '<!-- end~header~' . MX_TIME . '~pmx -->';
    }

    /**
     * pmxHeader::move()
     *
     * @param mixed $showed
     * @return
     */
    public static function move($showed)
    {
        $pattern = '#<!-- start~header~' . MX_TIME . '~pmx -->.*?<!-- end~header~' . MX_TIME . '~pmx -->#s';
        if (preg_match($pattern, $showed, $matches)) {
            $showed = str_replace($matches[0], self::get(), $showed);
        }
        return $showed;
    }

    /**
     * pmxHeader::fetch()
     * Einstellungen pruefen und komplettieren
     *
     * @param mixed $defaults
     * @return
     */
    public static function fetch($defaults = false)
    {
        $more = self::get_more(); // als erstes abfragen, weil in my_header noch Zuweisungen stehen knnten
        $export['style_code'] = self::get_style_code();
        $export['style'] = self::get_style();
        $export['style'] .= "\n" . '<link rel="stylesheet" href="' . MX_THEME_DIR . '/style/style.css" type="text/css" />' . "\n";
        $export['jquery'] = self::get_jquery();
        $export['script_code'] = self::get_script_code();
        $export['script'] = self::get_script();

        $export['more'] = $more;
        return $export;
    }

    /**
     * pmxHeader::add()
     * Adds code to output buffer
     *
     * @param string $code code for output
     * @param bool $extract
     * @return
     */
    public static function add($code, $extract = false)
    {
        if (is_array($code)) {
            foreach ($code as $item) {
                if ($extract) {
                    $item = self::_extract($item);
                }
                self::$__adds['more'][] = $item;
            }
        } else {
            if ($extract) {
                $code = self::_extract($code);
            }
            self::$__adds['more'][] = $code;
        }
    }

    /**
     * pmxHeader::add_style()
     *
     * @param mixed $src
     * @param string $media
     * @param string $if_for_ie
     * @return
     */
    public static function add_style($src, $media = '', $if_for_ie = '')
    {
        switch (true) {
            case !$src:
            case $if_for_ie && !self::_is_ie():
                return false;
            case $media:
                $media = ' media="' . $media . '"';
        }

        /* Systempfade am Anfang entfernen */
        $src = self::_cleanpath($src);

        /* verhindern, dass url doppelt eingefuegt wird */
        if (in_array($src, self::$__urls)) {
            return true;
        }
        self::$__urls[] = $src;

        $src = '<link rel="stylesheet" type="text/css" href="' . trim($src) . '"' . $media . ' />';
        if ($if_for_ie) {
            $src = "<!--[if " . $if_for_ie . "]>\n" . $src . "\n<![endif]-->";
        }

        self::$__adds['style'][] = $src;
    }

    /**
     * pmxHeader::add_style_code()
     *
     * @param mixed $code
     * @return
     */
    public static function add_style_code($code)
    {
        self::$__adds['style_code'][] = preg_replace(array('#</?style[^>]*>#i', '#[[:cntrl:]]#', '#\s+#'), array('', '', ' '), $code);
    }

    /**
     * pmxHeader::add_script()
     *
     * @param mixed $src
     * @param string $if_for_ie
     * @return
     */
    public static function add_script($src, $if_for_ie = '')
    {
        switch (true) {
            case !$src:
            case $if_for_ie && !self::_is_ie():
                return false;
        }

        /* Systempfade am Anfang entfernen */
        $src = trim(self::_cleanpath($src));

        /* verhindern, dass url doppelt eingefuegt wird */
        if (in_array($src, self::$__urls)) {
            return true;
        }

        $src = self::_getminfilename($src); // min Version vorhanden?

        self::$__urls[] = $src;

        if (strrpos($src, '/' . self::$__jquery['main']) !== false && !$if_for_ie) {
            self::add_jquery();
            return;
        }

        $src = '<script type="text/javascript" src="' . $src . '"></script>';
        if ($if_for_ie) {
            $src = "<!--[if " . $if_for_ie . "]>\n" . $src . "\n<![endif]-->";
        }

        self::$__adds['script'][] = $src;
    }

    /**
     * pmxHeader::add_script_code()
     *
     * @param mixed $code
     * @return
     */
    public static function add_script_code($code)
    {
        self::$__adds['script_code'][] = preg_replace('#</?script[^>]*>#i', '', $code);
    }

    /**
     * pmxHeader::add_jquery()
     *
     * @return
     */
    public static function add_jquery()
    {
        self::$__jquery['required'] = true;
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.core.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.accordion.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.autocomplete.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.button.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.datepicker.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.dialog.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.progressbar.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.resizable.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.selectable.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.slider.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.tabs.css');
        // pmxHeader::add_style('layout/jquery/ui/base/jquery.ui.theme.css');
        if (!self::$__jquery['loaded']) {
            self::$__jquery['loaded'] = true;
            $src = self::$__jquery['path'] . self::$__jquery['main'];

            $src = self::_getminfilename($src); // min Version vorhanden?
            array_unshift(self::$__jquery['files'], $src);
            /* verhindern, dass url doppelt eingefuegt wird */
            self::$__urls[] = $src;
        }

        $args = func_get_args();

        if ($args) {
            foreach ($args as $src) {
                $src = self::$__jquery['path'] . $src;

                $src = self::_getminfilename($src); // min Version vorhanden?
                /* verhindern, dass url doppelt eingefuegt wird */
                if (in_array($src, self::$__urls)) {
                    continue;
                }
                // TODO: hier noch check ob MIN Version vorhanden
                self::$__urls[] = $src;
                self::$__jquery['files'][] = $src;
            }
        }
    }

    /**
     * pmxHeader::add_lightbox()
     *
     * @param mixed $reset
     * @return nothing
     */
    public static function add_lightbox($reset = false)
    {
        $reset = ($reset) ? '?t=' . intval(MX_TIME) : '';
        self::add_style(PMX_LAYOUT_PATH . 'jquery/css/prettyPhoto.css');
        self::add_jquery('jquery.prettyPhoto.js', 'lightbox.js.php' . $reset);
    }

    /**
     * pmxHeader::add_prettyphoto()
     * clone von add_lightbox()
     *
     * @param mixed $reset
     * @return nothing
     */
    public static function add_prettyphoto($reset = false)
    {
        self::add_lightbox($reset);
    }

    /**
     * pmxHeader::add_tabs()
     *
     * @param mixed $cookie
     * @return nothing
     */
    public static function add_tabs($cookie = true)
    {
        self::add_jquery('ui/jquery-ui-pmx-core.min.js', 'ui/jquery.ui.tabs.min.js');
        if ($cookie) {
            self::add_jquery('jquery.cookie.js');
        }
    }

    /**
     * pmxHeader::get_style()
     *
     * @return
     */
    public static function get_style()
    {
        if (array_key_exists('style', self::$__adds)) {
            return implode("\n", array_unique(self::$__adds['style'])) . "\n";
        }
    }

    /**
     * pmxHeader::get_style_code()
     *
     * @return
     */
    public static function get_style_code()
    {
        if (array_key_exists('style_code', self::$__adds)) {
            return self::_preparelines(self::$__adds['style_code'], 'style') . "\n";
        }
    }

    /**
     * pmxHeader::get_script()
     *
     * @return
     */
    public static function get_script()
    {
        if (array_key_exists('script', self::$__adds)) {
            return implode("\n", array_unique(self::$__adds['script'])) . "\n";
        }
    }

    /**
     * pmxHeader::get_script_code()
     *
     * @return
     */
    public static function get_script_code()
    {
        if (array_key_exists('script_code', self::$__adds)) {
            return self::_preparelines(self::$__adds['script_code'], 'script') . "\n";
        }
    }

    /**
     * pmxHeader::get_jquery()
     *
     * @return
     */
    public static function get_jquery()
    {
        if (self::$__jquery['required']) {
            self::$__jquery['files'] = array_unique(self::$__jquery['files']);
            foreach (self::$__jquery['files'] as $addon) {
                $srcs[] = '<script type="text/javascript" src="' . $addon . '"></script>';
            }
            return implode("\n", $srcs) . "\n";
        }
        return;
    }

    /**
     * pmxHeader::get_more()
     *
     * @return
     */
    public static function get_more()
    {
        /* die my_header.php includen und bearbeiten */
        self::get_user();

        /* die Sammlung zurueckgeben... */
        if (array_key_exists('more', self::$__adds)) {
            $more = array_unique(self::$__adds['more']);
            foreach ($more as $key => $value) {
                $more[$key] = preg_replace('#\s*\n\s+#', "\n", $value);
            }
            return implode("\n", $more) . "\n";
        }
    }

    /**
     * pmxHeader::get_user()
     * u.A. die my_header.php includen und bearbeiten
     *
     * @return
     */
    public static function get_user()
    {
        /* wenn keine Datenbankverbindung mehr besteht, my_header.php nicht includen */
        if (!isset($GLOBALS['dbi']) || !is_resource($GLOBALS['dbi'])) {
            return;
        }

        /* zusaetzliche Datei fuer benutzerdefinierte Zusaetze im HTML-Headerbereich */
        if (!($my_header = realpath(PMX_SYSTEM_DIR . '/my_header.php'))) {
            return;
        }

        ob_start();
        if (include_once($my_header)) {
            $my_header = trim(ob_get_clean());
            if ($my_header) {
                self::add($my_header);
            }
        } else {
            ob_end_clean();
        }
    }

    /**
     * only DUMMY !!
     * pmxHeader::set_description()
     * set meta description
     *
     * @param mixed $description
     * @return
     */
    public static function set_description($description)
    {
        return;
    }

    /**
     * only DUMMY !!
     * pmxHeader::set_keywords()
     * set and replace existing keywords
     *
     * @param mixed $keywords
     * @return
     */
    public static function set_keywords($keywords)
    {
        return;
    }

    /**
     * pmxHeader::_preparelines()
     *
     * @param mixed $lines
     * @param mixed $type
     * @return
     */
    private static function _preparelines($lines, $type)
    {
        $lines = trim(implode("\n", array_unique($lines)));
        if (!$lines) {
            return '';
        }

        /* Kommentare entfernen */
        $lines = str_replace(array('<![CDATA[', ']]>'), '', $lines);
        $lines = preg_replace(array('#/\*[^\*]*\*/#sU', '#\<\!--.+--\>#siU', '#\n\s*//[^\n]*#'), '', $lines);

        if ($type === 'script') {
            $out = '<script type="text/javascript">';
        } else {
            $out = '<style type="text/css">';
        }

        $out .= "\n /* <![CDATA[ */\n" . $lines . "\n /* ]]> */\n";

        if ($type === 'script') {
            $out .= '</script>';
        } else {
            $out .= '</style>';
        }
        return $out;
    }

    /**
     * pmxHeader::_cleanpath()
     * schneidet Sytempfade am Anfang eines Pfades oder URL ab
     *
     * @param mixed $path
     * @return
     */
    private static function _cleanpath($path)
    {
        $path = mx_strip_sysdirs($path);
        return str_replace(DS, '/', $path);
    }

    /**
     * pmxHeader::_extract()
     * filtert bestimmte Bestandteile aus gemischten Headerdaten
     *
     * @param mixed $code
     * @return
     */
     
    public static function set_jquery($name, $value = null)
		 {
     if(null === $value && is_array($name)){
         self::$__jquery = $name;
     }elseif(isset(self::$__jquery[$name])){
         $__jquery[$name] = $value;
      }
     return self; 
  } 
     
    private static function _extract($code)
    {
        /* conditional comments ignorieren <!--[if lte IE 6]><![endif]--> */
        preg_match_all('#<\!--\s*\[if[^\]]+\]>.+<\!\[endif\]\s*-->#siU', $code, $matches);
        $condition = array();
        if ($matches) {
            if (!self::_is_ie()) {
                $code = str_replace($matches[0], '', $code);
            } else {
                foreach ($matches[0] as $key => $src) {
                    $condition['~!~' . $key . '~!~'] = $src;
                }
            }
            if ($condition) {
                $code = str_replace(array_values($condition), array_keys($condition), $code);
            }
        }

        /* Scriptdateien rausfiltern und an Scriptarray anfgen */
        preg_match_all('#<script[^>]+src\s*=\s*["\']([^"\']+)["\'][^>]*>\s*</script>#siU', $code, $matches);
        if ($matches) {
            $code = str_replace($matches[0], '', $code);
            foreach ($matches[1] as $src) {
                self::add_script($src);
            }
        }

        /* Script- und Style-Codedateien rausfiltern und an entspr. Array anfgen */
        preg_match_all('#<(script|style)[^>]*>(.+)</\1>#siU', $code, $matches);
        if ($matches) {
            $code = str_replace($matches[0], '', $code);
            foreach ($matches[2] as $key => $src) {
                if ($matches[1][$key] == 'script') {
                    self::add_script_code($src);
                } else {
                    self::add_style_code($src);
                }
            }
        }

        /* Stylesheets rausfiltern und an Sheetarray anfgen */
        preg_match_all('#<link[^>]+href\s*=\s*["\']([^"\']+)["\'][^>]*/?>#siU', $code, $matches);
        if ($matches) {
            foreach ($matches[1] as $key => $src) {
                if (strpos($matches[0][$key], 'stylesheet') !== false) {
                    self::add_style($src);
                    $code = str_replace($matches[0][$key], '', $code);
                }
            }
        }

        /* conditional comments wieder zurckschreiben */
        if ($condition && $code) {
            $code = str_replace(array_keys($condition), array_values($condition), $code);
        }

        if ($code) {
            /* falls xhtml-Endung fehlt, diese anfgen */
            $code = preg_replace('#<((?:img|input|hr|br|link|meta|base)(?:[^>]*[^/])?)>#i', '<$1 />', $code);
        }

        /* raus damit */
        return $code;
    }

    /**
     * pmxHeader::_is_ie()
     * guggen, was fr ein Browser am werkeln ist
     *
     * @return bool
     */
    private static function _is_ie()
    {
        $browser = load_class('Browser');
        return $browser->is_msie();
    }

    /**
     * pmxHeader::_getminfilename()
     *
     * @param string $src
     * @param string $ending
     * @return string $src
     */
    private static function _getminfilename($src, $ending = 'js')
    {
        if (pmxDebug::is_mode('enhanced') && is_file($src)) {
            // im erweiterten Debugmodus die Originale laden
            return $src;
        }
        $tmp = explode('.', $src);

        /* Endung wie gefordert? */
        if (array_pop($tmp) != $ending) {
            return $src;
        }

        /* bereits minimierte Version? */
        $last = array_pop($tmp);
        if ($last == 'min') {
            return $src;
        }

        /* Array ergnzen + min */
        array_push($tmp, $last, 'min', $ending);

        /* Dateiname aus Array wieder herstellen */
        $tmp = implode('.', $tmp);
        /* wenn minimierte Version vorhanden, diese verwenden */
        if (is_file($tmp)) {
            $src = $tmp;
        }
        return $src;
    }
}

?>
